function [stats df] = mglm(Y, X)

% Function mglm is an implementation of the multivariate linear model in Car package in R
% It uses Wilk's Lambda to approximate f-stat and corresponding p-value. 
%
% Multivariate Linear Model:
%   Y = BX + e 
%
% Input:
%     Y: response multivariate variable
%     X: model matrix
%     
% Output:
%     stats.F: F-stats
%     stats.p: p-value
%
% Won Hwa Kim - wonhwa@cs.wisc.edu
% Updated: Apr. 19, 2013
%

%% dimensions
n=size(Y,1);
k=size(X,2)+1; % constant
p=size(X,2);

%% Computing sum-of-squares
X2 = [ones(n,1) X];
temp = inv(X2'*X2)*X2'*Y;

intercept = temp(1,:);  % intercept
eB = temp(2:end,:); % B estimation

eY = X*eB;
eE = Y - repmat(intercept,n,1) - eY;    % residual

SSP_R = eE'*eE; % SSP residual matrix
y_mean = mean(Y - repmat(intercept,n,1))';  % mean of y
SSP_Reg = eY' * eY - n*y_mean*y_mean';
% SSP_T = SSP_R + SSP_Reg;

SSP_H = SSP_Reg;
%% Wilk's lambda 
WL = det(SSP_R) / det(SSP_H + SSP_R);

%% F-stat and p-value approximation 
m = size(SSP_R,1); 
s = p; 
r = n-k - (m-s+1)/2;
u = (m*s-2)/4;

if m^2 + s^2 - 5 > 0
    t = sqrt((m^2 * s^2 - 4) / (m^2 + s^2 - 5));
else
    t = 1;
end
df = [r*t-2*u, m*s];

F = (1-WL^(1/t))/ WL^(1/t) * (r*t - 2*u)/(m*s);
pval = 1-fcdf(F,df(2),df(1));

%% Output
stats.f = F;
stats.p = pval;
